import React from 'react';
import { matchMedia } from '@shopify/jest-dom-mocks';
import { mountWithContext } from '../../test-utilities';
import { UnstyledLink } from '../Link';
import { Spinner } from '../Spinner';
import { Button } from './Button';
var defaultProps = {
  children: 'Go to next step'
};
describe('<Button />', function () {
  beforeEach(function () {
    matchMedia.mock();
  });
  afterEach(function () {
    matchMedia.restore();
  });
  describe('`<button />`', function () {
    describe('children', function () {
      it('renders the children into the button', function () {
        var text = 'Go to shipping';
        var button = mountWithContext( /*#__PURE__*/React.createElement(Button, null, text));
        expect(button).toContainReactText(text);
      });
      it('renders the children into the link', function () {
        var text = 'Go to shipping';
        var linkButton = mountWithContext( /*#__PURE__*/React.createElement(Button, {
          to: "/shipping"
        }, text));
        expect(linkButton).toContainReactText(text);
      });
    });
    describe('onPress()', function () {
      it('is called when button is pressed', function () {
        var onPressSpy = jest.fn();
        var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
          onPress: onPressSpy
        })));
        button.find('button').trigger('onClick');
        expect(onPressSpy).toHaveBeenCalledTimes(1);
      });
    });
  });
  describe('`<UnstyledLink />`', function () {
    it('renders when a url is provided', function () {
      var url = '/shipping';
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        to: url
      })));
      expect(button).toContainReactComponent(UnstyledLink, {
        to: url
      });
    });
    it('renders a button when disabled', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        disabled: true,
        to: "/shipping"
      })));
      expect(button).not.toContainReactComponent(UnstyledLink);
      expect(button).toContainReactComponent('button', {
        disabled: true
      });
    });
  });
  describe('submit', function () {
    it('renders a type button when not present', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, defaultProps));
      expect(button).toContainReactComponent('button', {
        type: 'button'
      });
    });
    it('renders a type submit when present', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        submit: true
      })));
      expect(button).toContainReactComponent('button', {
        type: 'submit'
      });
    });
  });
  describe('loading', function () {
    it('disables button when loading', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true
      })));
      expect(button).toContainReactComponent('button', {
        disabled: true
      });
    });
    it('indicates that the button is getting updated', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true
      })));
      expect(button).toContainReactComponent('button', {
        'aria-busy': true
      });
    });
    it('renders a <Spinner> when loading', function () {
      matchMedia.setMedia(function () {
        return {
          matches: false
        };
      });
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loading: true
      })));
      expect(button).toContainReactComponent(Spinner);
    });
    it('renders the accessibleLabel as the Spinner children when provided', function () {
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loadingLabel: "Processing",
        loading: true
      })));
      expect(button).toContainReactComponent(Spinner, {
        children: 'Processing'
      });
    });
    it('adds the Spinner class when user does not prefers reduced motion', function () {
      matchMedia.setMedia(function () {
        return {
          matches: false
        };
      });
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loadingLabel: "Processing",
        loading: true
      })));
      expect(button).toContainReactComponent('span', {
        className: 'LoadingContent Spinner'
      });
    });
    it('removes the Spinner class when user prefers reduced motion', function () {
      matchMedia.setMedia(function () {
        return {
          matches: true
        };
      });
      var button = mountWithContext( /*#__PURE__*/React.createElement(Button, Object.assign({}, defaultProps, {
        loadingLabel: "Processing",
        loading: true
      })));
      expect(button).not.toContainReactComponent('span', {
        className: 'LoadingContent Spinner'
      });
    });
  });
});