"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const test_utilities_1 = require("../../test-utilities");
const Labelled_1 = require("./Labelled");
const hook_1 = require("./hook");
const context_1 = require("./context");
const defaultProps = {
    children: <div />,
    label: '',
    htmlFor: '',
    isEmpty: true,
};
describe('<Labelled />', () => {
    it('renders the children', async () => {
        const labelled = test_utilities_1.mountWithContext(<Labelled_1.Labelled {...defaultProps}>
        <Child />
      </Labelled_1.Labelled>);
        expect(labelled).toContainReactComponent(Child);
    });
    it('wraps children with a LabelledContext.Provider', async () => {
        const label = 'Email';
        const labelled = test_utilities_1.mountWithContext(<Labelled_1.Labelled {...defaultProps} label={label}/>);
        expect(labelled).toProvideReactContext(context_1.LabelledContext);
    });
    it('hides the floating label if no value is entered', async () => {
        const label = 'Email';
        const labelled = test_utilities_1.mountWithContext(<Labelled_1.Labelled {...defaultProps} label={label}/>);
        expect(labelled).toProvideReactContext(context_1.LabelledContext, expect.objectContaining({
            isFloating: false,
        }));
    });
    it('renders a floating label once a value is entered', async () => {
        const label = 'Email';
        const labelled = test_utilities_1.mountWithContext(<Labelled_1.Labelled {...defaultProps} label={label}>
        <Child />
      </Labelled_1.Labelled>);
        labelled.act(() => {
            var _a;
            (_a = labelled.find('input')) === null || _a === void 0 ? void 0 : _a.trigger('oninput', {
                target: {
                    value: 'snowdevil@shopify.com',
                },
            });
        });
        expect(labelled).toProvideReactContext(context_1.LabelledContext, expect.objectContaining({
            isFloating: true,
        }));
    });
    it('displays a floating label once a value is entered and removed', async () => {
        const label = 'Email';
        const labelled = test_utilities_1.mountWithContext(<Labelled_1.Labelled {...defaultProps} label={label}>
        <Child />
      </Labelled_1.Labelled>);
        // Enter value
        labelled.act(() => {
            var _a;
            (_a = labelled.find('input')) === null || _a === void 0 ? void 0 : _a.trigger('oninput', {
                target: {
                    value: 'snowdevil@shopify.com',
                },
            });
        });
        // Remove value
        labelled.act(() => {
            var _a;
            (_a = labelled.find('input')) === null || _a === void 0 ? void 0 : _a.trigger('oninput', {
                target: {
                    value: '',
                },
            });
        });
        expect(labelled).toProvideReactContext(context_1.LabelledContext, expect.objectContaining({
            isFloating: true,
        }));
    });
    it('hides a floating label once a value is entered, removed and focus is lost', async () => {
        const label = 'Email';
        const labelled = test_utilities_1.mountWithContext(<Labelled_1.Labelled {...defaultProps} label={label}>
        <Child />
      </Labelled_1.Labelled>);
        // Enter value
        labelled.act(() => {
            var _a;
            (_a = labelled.find('input')) === null || _a === void 0 ? void 0 : _a.trigger('oninput', {
                target: {
                    value: 'snowdevil@shopify.com',
                },
            });
        });
        // Remove value
        labelled.act(() => {
            var _a;
            (_a = labelled.find('input')) === null || _a === void 0 ? void 0 : _a.trigger('oninput', {
                target: {
                    value: '',
                },
            });
        });
        // Lose focus
        labelled.act(() => {
            var _a;
            (_a = labelled.find('input')) === null || _a === void 0 ? void 0 : _a.trigger('onBlur', {
                target: {
                    value: '',
                },
            });
        });
        expect(labelled).toProvideReactContext(context_1.LabelledContext, expect.objectContaining({
            isFloating: false,
        }));
    });
    it('displays a floating label once a value is entered and focus is lost', async () => {
        const label = 'Email';
        const labelled = test_utilities_1.mountWithContext(<Labelled_1.Labelled {...defaultProps} label={label}>
        <Child />
      </Labelled_1.Labelled>);
        // Enter value
        labelled.act(() => {
            var _a;
            (_a = labelled.find('input')) === null || _a === void 0 ? void 0 : _a.trigger('oninput', {
                target: {
                    value: 'snowdevil@shopify.com',
                },
            });
        });
        // Lose focus
        labelled.act(() => {
            var _a;
            (_a = labelled.find('input')) === null || _a === void 0 ? void 0 : _a.trigger('onBlur', {
                target: {
                    value: '',
                },
            });
        });
        expect(labelled).toProvideReactContext(context_1.LabelledContext, expect.objectContaining({
            isFloating: true,
        }));
    });
    // eslint-disable-next-line jest/no-disabled-tests
    it.skip('displays a floating label if the field is initially empty but its value gets updated', async () => {
        const asynchValue = '150 Elgin';
        const labelled = test_utilities_1.mountWithContext(<Labelled_1.Labelled {...defaultProps}>
        <Child value=""/>
      </Labelled_1.Labelled>);
        // TODO - Failing test, review how we want to test children with async values. Recent Preact changes skip setProperty for children and key props
        // might be the culprit of this specific behaviour. https://github.com/preactjs/preact/pull/2213
        labelled.act(() => {
            labelled.setProps({
                isEmpty: false,
                children: <Child value={asynchValue}/>,
            });
        });
        expect(labelled).toContainReactComponent('input', {
            value: asynchValue,
        });
        expect(labelled).toProvideReactContext(context_1.LabelledContext, expect.objectContaining({
            isFloating: true,
        }));
    });
});
function Child({ value }) {
    const { onBlur, onChange, onFocus } = hook_1.useLabelled();
    return (<input value={value} onBlur={onBlur} onChange={({ target }) => {
        onChange(!target.value || target.value === '');
    }} onFocus={onFocus}/>);
}
